

#    provide commands for CAMAC access using the Hytec VSD5992 interface

#    This package provides the commands rdCAMAC and wtCAMAC

#     This is a VME interface and so the HytecVSD2992 package uses commands rdVME and wtVME which
#     must be provided via the appropiate VME access package  (VMEAccess CAENV1718)

set VMEBaseAddress 0xc000    ;#   default module VME address in A16 space

set selClrLams [expr $VMEBaseAddress + 0x0e]
set txC        [expr $VMEBaseAddress + 0x18]
set txANF      [expr $VMEBaseAddress + 0x16]
set txDataTp   [expr $VMEBaseAddress + 0x1c]
set txDataBtm  [expr $VMEBaseAddress + 0x1a]
set lamStatus  [expr $VMEBaseAddress + 0x0e]
set rxDataTp   [expr $VMEBaseAddress + 0x1c]
set rxDataBtm  [expr $VMEBaseAddress + 0x1a]

set TimeOut 100
set QX(0) 3; set QX(1) 2; set QX(2) 1; set QX(3) 0        ;# to invert NOT QX bits

proc WaitForModule {} {
    global lamStatus TimeOut QX

   set loops 0

#    wait for the module RDY bit to be set

    while {$loops < $TimeOut} {
   
        set rc [rdVME -a $lamStatus -m 0x29 -w 16]   ;#   read LAM status
        if {[first $rc] != 0} then {puts "Error reading VSD2992 Status"; return [list -1 0]} ;#   error reading status
        if {[expr [second $rc] & 0x200] != 0}  {return [list 0 $QX([expr [second $rc] & 3])]}  ;#  RDY set 

        incr loops
    }

    puts "Timeout waiting for VSD2992; Status = $rc"
    return [list 1 0]      ;#  give up waiting 
}

proc stCAMAC {} {return 0}

proc rdCAMAC {args} {
    global selClrLams txC txANF rxDataTp rxDataBtm

    set c 1; set n 1; set a 0; set f 0

    for {set index 0} {$index < [llength $args]} {} {
        switch -- [lindex $args $index] {
            -c   {set c [lindex $args [expr $index + 1]]}
            -n   {set n [lindex $args [expr $index + 1]]}
            -a   {set a [lindex $args [expr $index + 1]]}
            -f   {set f [lindex $args [expr $index + 1]]}
            default {}
        }
        incr index 2
    }

    if {$f >= 16 && $f <= 23} {return -1}    ;#  function is set to a write operation

    wtVME -a $selClrLams -m 0x29 -w 16 -d -1   ;#  clear all LAMS
    wtVME -a $txC -m 0x29 -w 16 -d $c          ;#  write crate

    set ANF [expr (($n & 0x1f) << 9) | (($a & 0xf) << 5) | ($f & 0x1f)]
    wtVME -a $txANF -m 0x29 -w 16 -d $ANF      ;#  write a,n,f

#    the CAMAC cycle now starts - wait for completion

    set rc [WaitForModule]
    if {[first $rc] != 0} {return $rc}
    set qx [second $rc]

    set High [second [rdVME -a $rxDataTp -m 0x29 -w 16]]
    set Low  [second [rdVME -a $rxDataBtm -m 0x29 -w 16]]

    set data [expr (($High << 16) & 0xff0000) | ($Low & 0x00ffff)]

    return [list 0 $data $qx]
}

proc wtCAMAC {args} {
    global selClrLams txC txANF txDataTp txDataBtm

    set c 1; set n 1; set a 0; set f 0; set data 0

    for {set index 0} {$index < [llength $args]} {} {
        switch -- [lindex $args $index] {
            -c   {set c [lindex $args [expr $index + 1]]}
            -n   {set n [lindex $args [expr $index + 1]]}
            -a   {set a [lindex $args [expr $index + 1]]}
            -f   {set f [lindex $args [expr $index + 1]]}
            -d   {set data [lindex $args [expr $index + 1]]}
            default {}
        }
        incr index 2
    }

    if {$f <= 7} {return -1}    ;#  function is set to a read operation

    wtVME -a $selClrLams -m 0x29 -w 16 -d -1   ;#  clear all LAMS
    wtVME -a $txC -m 0x29 -w 16 -d $c          ;#  write crate

    set ANF [expr (($n & 0x1f) << 9) | (($a & 0xf) << 5) | ($f & 0x1f)]
    wtVME -a $txANF -m 0x29 -w 16 -d $ANF      ;#  write a,n,f

    wtVME -a $txDataTp -m 0x29 -w 16 -d [expr ($data >> 16) & 0xff]  ;# write top 8 data bits
    wtVME -a $txDataBtm -m 0x29 -w 16 -d [expr $data & 0x00ffff]     ;# write low 16 data bits

#    the CAMAC cycle now starts - wait for completion

    set rc [WaitForModule]
    return $rc
}

package provide HytecVSD2992 1.0
